/*********************************************************************
*          Portions COPYRIGHT 2016 STMicroelectronics                *
*          Portions SEGGER Microcontroller GmbH & Co. KG             *
*        Solutions for real time microcontroller applications        *
**********************************************************************
*                                                                    *
*        (c) 1996 - 2015  SEGGER Microcontroller GmbH & Co. KG       *
*                                                                    *
*        Internet: www.segger.com    Support:  support@segger.com    *
*                                                                    *
**********************************************************************

** emWin V5.32 - Graphical user interface for embedded applications **
All  Intellectual Property rights  in the Software belongs to  SEGGER.
emWin is protected by  international copyright laws.  Knowledge of the
source code may not be used to write a similar product.  This file may
only be used in accordance with the following terms:

The  software has  been licensed  to STMicroelectronics International
N.V. a Dutch company with a Swiss branch and its headquarters in Plan-
les-Ouates, Geneva, 39 Chemin du Champ des Filles, Switzerland for the
purposes of creating libraries for ARM Cortex-M-based 32-bit microcon_
troller products commercialized by Licensee only, sublicensed and dis_
tributed under the terms and conditions of the End User License Agree_
ment supplied by STMicroelectronics International N.V.
Full source code is available at: www.segger.com

We appreciate your understanding and fairness.
----------------------------------------------------------------------
File        : LCDConf_FlexColor_Template.c
Purpose     : Display controller configuration (single layer)
---------------------------END-OF-HEADER------------------------------
*/

/**
  ******************************************************************************
  * @attention
  *
  * Licensed under MCD-ST Liberty SW License Agreement V2, (the "License");
  * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at:
  *
  *        http://www.st.com/software_license_agreement_liberty_v2
  *
  * Unless required by applicable law or agreed to in writing, software 
  * distributed under the License is distributed on an "AS IS" BASIS, 
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
  ******************************************************************************
  */

#include "GUI.h"
#include "GUIDRV_FlexColor.h"
#include "main.h"
#include "gpio.h"
#include "TFT/ILI9328.h"
/*********************************************************************
*
*       Layer configuration (to be modified)
*
**********************************************************************
*/

//
// Physical display size
//
#define XSIZE_PHYS  240 // To be adapted to x-screen size
#define YSIZE_PHYS  320 // To be adapted to y-screen size



#define GUI_TOUCH_AD_TOP 	 				250
#define GUI_TOUCH_AD_BOTTOM  			3807
#define GUI_TOUCH_AD_LEFT 	 			3899
#define GUI_TOUCH_AD_RIGHT 	 			376


#define LCD_REG_ADDRESS		  *(__IO uint16_t *)(0x60000000)
#define LCD_DATA_ADDRESS		*(__IO uint16_t *)(0x60080000)


/*********************************************************************
*
*       Configuration checking
*
**********************************************************************
*/
#ifndef   VXSIZE_PHYS
  #define VXSIZE_PHYS XSIZE_PHYS
#endif
#ifndef   VYSIZE_PHYS
  #define VYSIZE_PHYS YSIZE_PHYS
#endif
#ifndef   XSIZE_PHYS
  #error Physical X size of display is not defined!
#endif
#ifndef   YSIZE_PHYS
  #error Physical Y size of display is not defined!
#endif
#ifndef   GUICC_565
  #error Color conversion not defined!
#endif
#ifndef   GUIDRV_FLEXCOLOR
  #error No display driver defined!
#endif

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/
/********************************************************************
*
*       LcdWriteReg
*
* Function description:
*   Sets display register
*/
static void LcdWriteReg(U16 Data) {
  // ... TBD by user
	LCD_REG_ADDRESS=Data;
}

/********************************************************************
*
*       LcdWriteData
*
* Function description:
*   Writes a value to a display register
*/
static void LcdWriteData(U16 Data) {
  // ... TBD by user
	LCD_DATA_ADDRESS=Data;
}

/********************************************************************
*
*       LcdWriteDataMultiple
*
* Function description:
*   Writes multiple values to a display register.
*/
static void LcdWriteDataMultiple(U16 * pData, int NumItems) {
  while (NumItems--) {
    // ... TBD by user
		LCD_DATA_ADDRESS=*pData++;
  }
}

/********************************************************************
*
*       LcdReadDataMultiple
*
* Function description:
*   Reads multiple values from a display register.
*/
static void LcdReadDataMultiple(U16 * pData, int NumItems) {
	*pData = LCD_DATA_ADDRESS;
  while (NumItems--) {
    // ... TBD by user
		*pData++=LCD_DATA_ADDRESS;
  }
}

/*********************************************************************
*
*       Public functions
*
**********************************************************************
*/
/*********************************************************************
*
*       LCD_X_Config
*
* Function description:
*   Called during the initialization process in order to set up the
*   display driver configuration.
*
*/
void LCD_X_Config(void) {
  GUI_DEVICE * pDevice;
	
  CONFIG_FLEXCOLOR Config = {0};
  GUI_PORT_API PortAPI = {0};
  //
  // Set display driver and color conversion
  //
  pDevice = GUI_DEVICE_CreateAndLink(GUIDRV_FLEXCOLOR, GUICC_565, 0, 0);
  //
  // Display driver configuration, required for Lin-driver
  //
  LCD_SetSizeEx (0, XSIZE_PHYS , YSIZE_PHYS);
  LCD_SetVSizeEx(0, VXSIZE_PHYS, VYSIZE_PHYS);
  //
  // Orientation
  //
  //Config.Orientation = GUI_SWAP_XY | GUI_MIRROR_Y|GUI_MIRROR_X;
	Config.Orientation = GUI_SWAP_XY|GUI_MIRROR_X;
  GUIDRV_FlexColor_Config(pDevice, &Config);
	
	GUI_TOUCH_SetOrientation((GUI_MIRROR_X * LCD_GetMirrorX())|(GUI_MIRROR_Y * LCD_GetMirrorY())|(GUI_SWAP_XY * LCD_GetSwapXY()));
	//
	// Calibrate touch screen
	//
	GUI_TOUCH_Calibrate(GUI_COORD_X, 0, 319, GUI_TOUCH_AD_TOP , GUI_TOUCH_AD_BOTTOM);
	GUI_TOUCH_Calibrate(GUI_COORD_Y, 0, 239, GUI_TOUCH_AD_LEFT, GUI_TOUCH_AD_RIGHT);
  //
  // Set controller and operation mode
  //
  PortAPI.pfWrite16_A0  = LcdWriteReg;
  PortAPI.pfWrite16_A1  = LcdWriteData;
  PortAPI.pfWriteM16_A1 = LcdWriteDataMultiple;
  PortAPI.pfReadM16_A1  = LcdReadDataMultiple;
  GUIDRV_FlexColor_SetFunc(pDevice, &PortAPI, GUIDRV_FLEXCOLOR_F66708, GUIDRV_FLEXCOLOR_M16C0B16);
}

/*********************************************************************
*
*       LCD_X_DisplayDriver
*
* Function description:
*   This function is called by the display driver for several purposes.
*   To support the according task the routine needs to be adapted to
*   the display controller. Please note that the commands marked with
*   'optional' are not cogently required and should only be adapted if
*   the display controller supports these features.
*
* Parameter:
*   LayerIndex - Index of layer to be configured
*   Cmd        - Please refer to the details in the switch statement below
*   pData      - Pointer to a LCD_X_DATA structure
*
* Return Value:
*   < -1 - Error
*     -1 - Command not handled
*      0 - Ok
*/
void emwin_WriteReg(uint16_t emwin_Reg,uint16_t emwin_RegValue)
{	
	LCD_REG_ADDRESS=emwin_Reg;
	LCD_DATA_ADDRESS=emwin_RegValue;	 
}
uint16_t emwin_RD_DATA(void)
{
	volatile uint16_t ram;			//ֹŻ
	ram=LCD_DATA_ADDRESS;	
	return ram;	 
}	
uint16_t emwin_ReadReg(uint16_t emwin_Reg)
{										   
	LcdWriteReg(emwin_Reg);		//дҪļĴ
	return emwin_RD_DATA();		//ضֵ
}

void emwin_Init()
{
	static uint16_t emwin_ID;
	emwin_ID=emwin_ReadReg(0x0000);	//ID9320/9325/9328/4531/4535IC   
  if(emwin_ID<0XFF||emwin_ID==0XFFFF||emwin_ID==0X9300)//IDȷ,emwin_ID==0X9300жϣΪ9341δλ»ᱻ9300
	{	
 		//9341 IDĶȡ		
		LcdWriteReg(0XD3);				   
		emwin_ID=emwin_RD_DATA();	//dummy read 	
 		emwin_ID=emwin_RD_DATA();	//0X00
  		emwin_ID=emwin_RD_DATA();   	//ȡ93								   
 		emwin_ID<<=8;
		emwin_ID|=emwin_RD_DATA();  	//ȡ41 	   			   
 		if(emwin_ID!=0X9341)		//9341,ǲ6804
		{	
 			LcdWriteReg(0XBF);				   
			emwin_ID=emwin_RD_DATA(); 	//dummy read 	 
	 		emwin_ID=emwin_RD_DATA();   	//0X01			   
	 		emwin_ID=emwin_RD_DATA(); 	//0XD0 			  	
	  		emwin_ID=emwin_RD_DATA();	//0X68 
			emwin_ID<<=8;
	  		emwin_ID|=emwin_RD_DATA();	//0X04	  
			if(emwin_ID!=0X6804)		//Ҳ6804,ԿǲNT35310
			{ 
				LcdWriteReg(0XD4);				   
				emwin_ID=emwin_RD_DATA();//dummy read  
				emwin_ID=emwin_RD_DATA();//0X01	 
				emwin_ID=emwin_RD_DATA();//0X53	
				emwin_ID<<=8;	 
				emwin_ID|=emwin_RD_DATA();	//0X10	 
				if(emwin_ID!=0X5310)		//ҲNT35310,ԿǲNT35510
				{
					LcdWriteReg(0XDA00);	
					emwin_ID=emwin_RD_DATA();		//0X00	 
					LcdWriteReg(0XDB00);	
					emwin_ID=emwin_RD_DATA();		//0X80
					emwin_ID<<=8;	
					LcdWriteReg(0XDC00);	
					emwin_ID|=emwin_RD_DATA();		//0X00		
					if(emwin_ID==0x8000)emwin_ID=0x5510;//NT35510صID8000H,Ϊ,ǿΪ5510
					if(emwin_ID!=0X5510)			//ҲNT5510,ԿǲSSD1963
					{
						LcdWriteReg(0XA1);
						emwin_ID=emwin_RD_DATA();
						emwin_ID=emwin_RD_DATA();	//0X57
						emwin_ID<<=8;	 
						emwin_ID|=emwin_RD_DATA();	//0X61	
						if(emwin_ID==0X5761)emwin_ID=0X1963;//SSD1963صID5761H,Ϊ,ǿΪ1963
					}
				}
			}
 		}  	
	} 
if(emwin_ID==0x9328)//ILI9328   OK  
	{
  		emwin_WriteReg(0x00EC,0x108F);// internal timeing      
 		emwin_WriteReg(0x00EF,0x1234);// ADD        
		//emwin_WriteReg(0x00e7,0x0010);      
        //emwin_WriteReg(0x0000,0x0001);//ڲʱ
        emwin_WriteReg(0x0001,0x0100);     
        emwin_WriteReg(0x0002,0x0700);//Դ                    

        emwin_WriteReg(0x0003,(1<<12)|(3<<4)|(0<<3) );//65K    
        emwin_WriteReg(0x0004,0x0000);                                   
        emwin_WriteReg(0x0008,0x0202);	           
        emwin_WriteReg(0x0009,0x0000);         
        emwin_WriteReg(0x000a,0x0000);//display setting         
        emwin_WriteReg(0x000c,0x0001);//display setting          
        emwin_WriteReg(0x000d,0x0000);//0f3c          
        emwin_WriteReg(0x000f,0x0000);
		//Դ
        emwin_WriteReg(0x0010,0x0000);   
        emwin_WriteReg(0x0011,0x0007);
        emwin_WriteReg(0x0012,0x0000);                                                                 
        emwin_WriteReg(0x0013,0x0000);                 
     	emwin_WriteReg(0x0007,0x0001);                 
       	HAL_Delay(50); 
        emwin_WriteReg(0x0010,0x1490);   
        emwin_WriteReg(0x0011,0x0227);
        HAL_Delay(50); 
        emwin_WriteReg(0x0012,0x008A);                  
        HAL_Delay(50); 
        emwin_WriteReg(0x0013,0x1a00);   
        emwin_WriteReg(0x0029,0x0006);
        emwin_WriteReg(0x002b,0x000d);
        HAL_Delay(50); 
        emwin_WriteReg(0x0020,0x0000);                                                            
        emwin_WriteReg(0x0021,0x0000);           
		HAL_Delay(50); 
		//٤У
        emwin_WriteReg(0x0030,0x0000); 
        emwin_WriteReg(0x0031,0x0604);   
        emwin_WriteReg(0x0032,0x0305);
        emwin_WriteReg(0x0035,0x0000);
        emwin_WriteReg(0x0036,0x0C09); 
        emwin_WriteReg(0x0037,0x0204);
        emwin_WriteReg(0x0038,0x0301);        
        emwin_WriteReg(0x0039,0x0707);     
        emwin_WriteReg(0x003c,0x0000);
        emwin_WriteReg(0x003d,0x0a0a);
        HAL_Delay(50); 
        emwin_WriteReg(0x0050,0x0000); //ˮƽGRAMʼλ 
        emwin_WriteReg(0x0051,0x00ef); //ˮƽGRAMֹλ                    
        emwin_WriteReg(0x0052,0x0000); //ֱGRAMʼλ                    
        emwin_WriteReg(0x0053,0x013f); //ֱGRAMֹλ  
 
        emwin_WriteReg(0x0060,0xa700);        
        emwin_WriteReg(0x0061,0x0001); 
        emwin_WriteReg(0x006a,0x0000);
        emwin_WriteReg(0x0080,0x0000);
        emwin_WriteReg(0x0081,0x0000);
        emwin_WriteReg(0x0082,0x0000);
        emwin_WriteReg(0x0083,0x0000);
        emwin_WriteReg(0x0084,0x0000);
        emwin_WriteReg(0x0085,0x0000);
      
        emwin_WriteReg(0x0090,0x0010);     
        emwin_WriteReg(0x0092,0x0600);  
        //ʾ    
        emwin_WriteReg(0x0007,0x0133); 
	}
	HAL_GPIO_WritePin(LCD_BL_GPIO_Port,LCD_BL_Pin,GPIO_PIN_SET);
	//GUI_SetBkColor(GUI_BLACK);
	//GUI_Clear();
}
int LCD_X_DisplayDriver(unsigned LayerIndex, unsigned Cmd, void * pData) {
  int r;
  (void) LayerIndex;
  (void) pData;
  
  switch (Cmd) {
  case LCD_X_INITCONTROLLER: {
    //
    // Called during the initialization process in order to set up the
    // display controller and put it into operation. If the display
    // controller is not initialized by any external routine this needs
    // to be adapted by the customer...
    //
    // ...
		emwin_Init();
		//TFT_Init();
    return 0;
  }
  default:
    r = -1;
  }
  return r;
}

/*************************** End of file ****************************/

